/***********************************************
                ACTION DEFINITION

  Actions are invisible objects created by
  designers to add a verb to a room.  A basic
  action does nothing more than generate a
  description of itself.  Custom actions that do
  something more complicated may be derived from
  it, however.
************************************************/

action
	parent_type = /atom/movable
	visibility = 0

	var
		udesc  //user description
		odesc  //description seen by others at final location
		//desc contains the description seen by all at original location
		hidden //suppress display of action in room panel

	proc
		SetName(T as text)
			//rename the verb
			var/v = verbs[1] //rename the verb
			verbs.len = 0       //clear the verb list
			new v(src,T)
			name = T
		Action()
			if(desc) view(src) << "[usr] [desc]"    //to all at original location
			if(udesc) usr << udesc                  //to user
			if(odesc) oview(usr) << "[usr] [odesc]" //to others at final location
		PostRead() //called after mud is loaded to do any final processing that could not be done in Read()
	VarIsEditable(varname)
		switch(varname)
			if("visibility","luminosity","opacity","density","gender","suffix") return 0
		return ..()
	Configure(property)
		switch(property)
			if("name")
				var/N = input("Enter a new name","name",name) as text|null
				if(isnull(N)) return
				SetName(N)
				return 1
			else return ..()
	verb/action()
		set src=view()
		set visibility = 1
		Action()
	Click()
		Action()
	VarDisplayName(V)
		switch(V)
			if("desc") return "desc (to all)"
			if("udesc") return "udesc (to user)"
			if("odesc") return "odesc (to others)"
		return ..()
	Read(savefile/F)
		..()
		SetName(name) //restore name of action verb

	exit
		var
			tmp/room/dest
			blind  //suppress display of destination in room panel
		teleport/blind = 1
		Action()
			if(usr.Move(dest)) ..()
		VarDisplayName(V)
			switch(V)
				if("desc") return "desc (departure)"
				if("udesc") return "udesc (to user)"
				if("odesc") return "odesc (arrival)"
			return ..()
		Write(savefile/F)
			..()
			F["dest"] << dest.roomid
		Read(savefile/F)
			..()
			F["dest"] >> dest
		PostRead()
			dest = mud.LocateRoom(dest)
		Configure(property)
			switch(property)
				if("dest")
					var/room/new_exit
					var/room/old_exit = vars[property]
					var/old_exit_id
					if(old_exit) old_exit_id = old_exit.roomid
					new_exit = input("Enter a room id (0 for none) or walk to the desired room and enter -1.",property,old_exit_id) as num|null
					if(isnull(new_exit)) return //canceled
					if(new_exit == -1) new_exit = usr.loc
					else if(new_exit == 0) new_exit = null
					else
						new_exit = mud.LocateRoom(new_exit)
						if(!new_exit)
							usr << "No such room."
							return

					vars[property] = new_exit
					return 1
				else return ..()

	logout
		Action()
			..()
			mud.StorePlayer(usr)
	export
		var
			address
			password
		Action()
			if(mud.ExportPlayer(usr,password,address))
				..()
				del(usr)
			else
				usr << "The gateway is down at the moment."

	import
		//This is not really an action, but it's convenient to define it here.
		//Create one of these objects for each different address and/or password
		//that you want to allow to send you player files.
		var
			password
			address
		hidden = 1
		New()
			verbs.Cut()
		VarIsEditable(varname)
			switch(varname)
				if("password","address","name") return 1
