
/***************************************
        DESIGNER GENES DEFINITION

This object gives its possessor all the verbs
necessary to build or modify the world.
****************************************/

mob/IsDesigner()
	if(locate(/obj/designer_genes) in usr) return 1
	if(locate(/obj/recessive_genes) in usr) return 1

obj/designer_genes
	name = "designer genes"
	gender = "plural"

	verb
		saveworld(Fname as text|null)
			set category = "Designer"
			set desc = "(filename) save world to default file if none specified"
			set src=usr.contents
			if(!Fname) mud.Write()
			else
				var/savefile/F = new(Fname)
				mud.Write(F)
				del(F)

		remove_player(key in mud.SavedPlayers())
			set category = "Designer"
			set desc = "(key) remove a player's savefile"
			set src=usr.contents
			mud.RemovePlayer(key)

		new_room(roomtype in mud.roomtypes,Name as text|null)
			set category = "Designer"
			set desc = "(type,name) create a new room."
			set src=usr.contents
			var/room/a = new roomtype()
			a.roomid = mud.NewRoomId()
			usr << "Created room number [a.roomid]."
			if(Name) a.name = Name
			a.DesignerInfoForm()

		goto_room(roomid as num)
			set category = "Designer"
			set src=usr.contents
			usr.Move(mud.LocateRoom(roomid))

		give(atom/movable/O,atom/movable/dst)
			set category = "Designer"
			set src=usr.contents
			O.Move(dst)

		edit(atom/O=null)
			set category = "Designer"
			set src=usr.contents
			set desc = "get designer information about an object"
			if(!O) O = usr.loc
			O.DesignerInfoForm()

		new_action(type in mud.actiontypes,Name as text|null)
			set category = "Designer"
			set src=usr.contents
			set desc = "(type,name) add an action to the current room"
			var/action/a = new type(usr.loc)
			if(Name) a.SetName(Name)
			a.DesignerInfoForm()

		new_object(atom/movable/P in mud.poptypes,Name as text|null)
			set category = "Designer"
			set src=usr.contents
			set desc = "(type,name) make an object from a population type"
			var/atom/movable/np = P.Clone(usr)
			np.poptype = null
			np.poploc = null
			if(Name) np.name = Name
			np.DesignerInfoForm()

		new_poptype(atom/movable/O in usr)
			set category = "Designer"
			set src=usr.contents
			set desc = "(object) make a new population type from an existing object"
			O.poptype = null
			O.poploc = null
			O.loc = null
			mud.poptypes.Add(O)

		add_pop(P in mud.poptypes)
			set category = "Designer"
			set src=usr.contents
			set desc = "(type) add a population entry to the current room"
			usr.loc:AddPop(P)

		del_pop(P in usr.loc:population)
			set category = "Designer"
			set src=usr.contents
			set desc = "remove a population entry from the current room"
			usr.loc:DelPop(P)

		repop()
			set category = "Designer"
			set src=usr.contents
			set desc = "repop the current room"
			usr.loc:Repop()

		destroy(O in view())
			set category = "Designer"
			set src=usr.contents
			set desc = "destroy on object"
			del(O)

		seeall()
			set category = "Designer"
			set src=usr.contents
			if(usr.sight & SEEINVIS)
				usr.sight &= ~SEEINVIS
				view() << "[usr]'s eyes stop glowing."
			else
				usr.sight |= SEEINVIS
				view() << "[usr]'s eyes glow strangely."

	proc/WelcomePlayer(mob/P)
		P << "Greetings, oh <a href=?\ref[src]help>creator</a> of worlds!"

	Topic(href)
		if(!(src in usr)) return
		if(href == "help")
			usr << "You have an object called 'designer genes'.  That gives you the power to edit the world at run-time.  The <b>Designer</b> panel lists the editing commands."

	New()
		. = ..()
		if(ismob(loc)) WelcomePlayer(loc)

//handy object for carrying designer genes without having all the extra verbs
obj/recessive_genes
	name = "designer genes"
	verb/activate()
		set category = "Designer"
		set desc = "re-enable the designer genes"
		new/obj/designer_genes(loc)
		del(src)
	/obj/designer_genes/verb/deactivate()
		set category = "Designer"
		set desc = "temporarily dissable the designer genes"
		new/obj/recessive_genes(loc)
		del(src)
