/********************************************
          OBJ and MOB DEFINITION

 These are merely basic objects without many
 properties.  The external code should derive
 additional objects from them.
*********************************************/

datum
	proc
		DesignerInfo()      //look at object properties
		DesignerInfoForm()  //includes header and footer around DesignerInfo
		GetDesignerText()   //text containing a link to this object
		Configure(property) //prompt for value of specified property
		VarIsEditable(varname)
		VarDisplayName(varname)   //variable name to show user in DesignerInfo()

atom/movable
	var/tmp
		atom/movable/poptype //object this one got cloned from
		room/poploc          //location where this object was cloned
	proc
		Clone(loc)      //make a copy of this object
		Repop()         //called when object was created during repop

mob
	proc/IsDesigner()
		//returns true if player has designer powers
		//defined in designer.dm

proc/DesignerText(O)
	if(istype(O,/datum)) return O:GetDesignerText(O)
	return O


atom/movable/Del()
	if(istype(poploc)) poploc.Unpop(src)
	return ..()

atom/movable/Clone(loc)
	var/obj/N = new type(loc)
	var/atom/movable/O
	N.poptype = src
	N.poploc = loc
	N.name = name
	N.desc = desc

	for(O in N.contents) del(O) //just in case
	for(O in contents)
		O = O.Clone(N)
	return N

datum/VarIsEditable(varname)
	switch(varname)
		if("overlays","underlays","icon","icon_state","layer","dir",
		   "suffix","text","sight","contents","tag","luminosity","density",
			"opacity")
			return 0

	return issaved(vars[varname])

obj/VarIsEditable(varname)
	switch(varname)
		if("suffix") return 1
	return ..()

datum/Topic(href)
	var/prm[] = params2list(href)

	if(!usr.IsDesigner())
		usr << "You do not currently have designer powers!"
		return

	if("Configure" in prm)
		if(Configure(prm["Configure"]))
			mud.dirty = 1
			DesignerInfoForm()
	else if("DesignerInfo" in prm)
		DesignerInfoForm()
	else return ..()

datum/Configure(property)
	if(!VarIsEditable(property) || !(property in vars))
		world.log << "Invalid call to [type]/Configure([property])."
		return

	var/val = vars[property]
	val = input(usr,"Enter a new value for '[property]'.",property,val) as null

	if(isnull(val)) return
	vars[property] = val
	return 1

datum/GetDesignerText()
	return "<a href=?\ref[src]DesignerInfo>[type]</a>"
atom/GetDesignerText()
	return "<a href=?\ref[src]DesignerInfo>[name]</a>"
atom/movable/GetDesignerText()
	if(poptype)	return "<a href=?\ref[src]DesignerInfo>[name]</a> (alive)"
	return ..()

datum/DesignerInfoForm()
	DesignerInfo()
	usr << "<HR>"

datum/VarDisplayName(V)
	return V

datum/DesignerInfo()
	var/V
	usr << "type: [type]"
	for(V in vars)
		if(VarIsEditable(V))
			usr << "<a href=?\ref[src]Configure=[V]>[VarDisplayName(V)]</a>: [DesignerText(vars[V])]"

atom/movable/DesignerInfo()
	var/atom/O
	if(poptype) usr << "poptype: [poptype.name] ([type])"
	..()

	var/count
	for(O in src)
		if(count++)
			usr << ", \..."
		else
			usr << "inventory: \..."
		usr << "[O.GetDesignerText()]\..."
	if(count)
		usr << ""
